from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect

from spread_dashboard.project import models
from spread_dashboard.responses import Error
from .models import CommissioningStrategy, StrategyForm


def index(request):
    return render(request, 'commissioning/editor/index.html', context={
        'title_1_href': f'commissioning/',
        'locations': models.Location.as_tree(),
        'projectName': models.Header.objects.get().name,
    })


@login_required
def of_location(request, location_id: int):
    if request.method == 'POST':
        for key, file in request.FILES.items():
            save_preview(f'/usr/local/spread/dashboard/static/commissioning/preview/{location_id}.jpg', file)

        return redirect(request.path)

    try:
        loc = models.Location.objects.get(id=location_id)
        navigations = (
            {
                'id': parent_id,
                'name': models.Location.objects.get(id=parent_id).name
            }
            for parent_id in loc.navigations
        )

        return render(request, 'commissioning/editor/location.html', context={
            'title_1_href': f'commissioning/{location_id}/',
            'navigations': navigations,
            'location': {
                'name': loc.name,
                'preview': f'commissioning/preview/{location_id}.jpg',
            }
        })

    except models.Location.DoesNotExist:
        return Error('location <%s> does not exist' % location_id)


@login_required
def of_area(request, location_id: int, area_id: int):
    if request.method == 'POST':
        if 'file' in request.FILES:
            for key, file in request.FILES.items():
                save_preview(f'/usr/local/spread/dashboard/static/commissioning/preview/{location_id}_{area_id}_{request.POST.get("index")}.jpg', file)

        if 'strategy' in request.POST:
            form = StrategyForm(request.POST, instance=CommissioningStrategy.objects.get(area_id=area_id))
            if form.is_valid():
                if form.changed_data:
                    form.save()

        return redirect(request.path)

    try:
        loc = models.Location.objects.get(id=location_id)
        area = models.Enginery.objects.get(id=area_id)

        navigations = (
            *(
                {
                    'id': parent_id,
                    'name': models.Location.objects.get(id=parent_id).name
                }
                for parent_id in loc.navigations
            ),
            {
                'id': area_id,
                'name': area.v_name
            }
        )

        strategy = StrategyForm(instance=CommissioningStrategy.objects.get_or_create(area_id=area_id)[0])

        return render(request, 'commissioning/editor/area.html', context={
            'title_1_href': f'commissioning/{location_id}/{area_id}',
            'navigations': navigations,
            'area': {
                'name': area.v_name,
                'preview_lights': f'commissioning/preview/{location_id}_{area_id}_1.jpg',
                'preview_sensors': f'commissioning/preview/{location_id}_{area_id}_2.jpg',
                'strategy': strategy
            }
        })

    except models.Subginery.DoesNotExist:
        return Error('Subginery does not exist')

    except models.Location.DoesNotExist:
        return Error('location <%s> does not exist' % location_id)


def save_preview(name, file):
    with open(name, 'wb') as dest:
        if file.multiple_chunks:
            for c in file.chunks():
                dest.write(c)
        else:
            dest.write(file.read())

        dest.close()
