var stack = [];

const now = new Date();
$(document).find('input[name=timeFrom]').val(now.toISOString().substr(0, 8) + '01');
$(document).find('input[name=timeTo]').val(now.toISOString().substr(0, 10));

const client = init_client(
    broker,
    function onConnect() {
        console.log('MQTT Connected');
        client.subscribe(broker['reply'], {});
        $('.state').each((i, node) => {
            client.subscribe(node.id, {});
        });
        setTimeout(getReport, 1000);
    },
    function onMessage(topic, message) {
        if (topic.startsWith('Spread/State')) {
            const targetElement = document.getElementById(topic);
            setValue(targetElement, message.value);

        } else if (message.value && message.value.data) {
            try {
                const consumption = parseFloat(message.value.data.value || 0) / 1000;
                const workTime = parseFloat(message.value.data.workTime || 0);

                const targetElement = document.getElementById(message.key);

                if (targetElement.hasAttribute('partof')) {
                    const amountElement = document.getElementById(targetElement.getAttribute('partof'));
                    const amountValue = parseFloat(amountElement.textContent) || 0;
                    const hasTargetValue = parseFloat(targetElement.textContent) || 0;
                    amountElement.textContent = (amountValue - hasTargetValue + consumption).toFixed(1);
                }

                targetElement.textContent = consumption.toFixed(1);

                $('#worktime-' + message.key).text((workTime / 3600000).toFixed(1));
            } catch (e) {
                console.error(e);
            } finally {
                request();
            }
        }
    }
);

function getReport() {
    try {
        const t1 = $(document).find('input[name=timeFrom]').val();
        const t2 = $(document).find('input[name=timeTo]').val();

        if (!t1 || !t2){
            return
        }

        if (stack.length !== 0) {
            alert(gettext('consumption_double_query'));
        }
        else {
            startTime = t1 + 'T00:00:00';
            endTime = t2 + 'T23:59:59';
            console.log('Query chart for period {} .. {}'
                .replace('{}', t1)
                .replace('{}', t2));

            $(document).find('.chartable').each((i, node) => {
                try {
                    stack.push(node);
                } catch (e) {
                    console.error(e);
                }
            });

            request();
        }

    } catch (e) {
        console.error(e);
    }

    return false;
}

function request() {
    if (stack.length > 0) {
        const element = stack.shift();
        const topic = element.getAttribute('chart');
        if (topic) {
            client.send(topic, JSON.stringify({
                params: {
                    algorithm: "integral",
                    startTime: startTime,
                    endTime: endTime,
                },
                key: element.id,
                session: broker.guid
            }));
        } else {
            request();
        }
    }
}

function setValue(node, value) {
    if ($(node).hasClass('profiles')) {
        for (var profile in value) {
            if (value.hasOwnProperty(profile)) {
                for (var funit in value[profile]) {
                    $(node)
                        .find('#' + profile)
                        .find('[name=' + funit + ']')
                        .val(value[profile][funit]);
                }
            }
        }
    } else if ($(node).is("select,input")) {
        $(node).val(value);
    } else {
        node.textContent = value;
    }
}